''      By Shawn A.
''
'' This is just a basic program to read the Gyro readings from a ITG3200 3 axis Gyro and convert the X and Y axis to an angle.
'' Then convert the Floating Point values to integers to display in PST.
'' The output angles will drift even when the board is lying flat and still on a table.  As much as 1 to 2 degrees a minute so don't freak out.
'' Jasons code zeros the Gyro on startup.
'' I am using a 6DOF board from SparkFun

CON
        _clkmode = xtal1 + pll16x
        _xinfreq = 5_000_000

MHZ   = 80
OHund = 100.0
GyroLSB = 14.375
GyroDT  = 0.004   'Is the duration of the repeat loop in the main program below.  This is 250Hz.  If this is changed the waitcnt command needs
                  'changed at the bottom of the repeat loop.
OBJ
FP32F    : "Float32Full"               'By Cam Thompson
Gyro     : "ITG 3200 Sport pasm"       'By Jason Dorie                      
PST      : "Parallax Serial Terminal"  'By Jeff Martin, Andy Lindsay, Chip Gracey

VAR
  Long Roll, Pitch              'Stores Roll and Pitch Angles as Floating Point.
  Long RollI,PitchI             'Stores Roll and Pitch Angles as Integer.
  Long Gx, Gy, Gz               'Gyro readings from ITG3200 
  Long LastTime                 'Used to keep track of 250Hz Sample Rate
  Long DisplayCounter           'Used to determine when to send data to PST
PUB Main

''''''Initializes a Couple of Things''''''
  PST.Start(115_200)        'Sets BaudRate for PST.
  PST.Clear                 'Clears PST Screen.
  FP32F.Start               'Starts Math Engine FF32.
  Gyro.Start(6,7)           'Starts Gyro Sets I2C Pins and Zeros the Gyro.
  DisplayCounter := 0
  LastTime       := cnt
''''''Since we are using a Gyro to find Angles of X and Y we need to time the repeat loop. At the end of the loop
''''''there is a waitcnt command that makes sure the loop takes 4mS, which is 250Hz.  We have to know how often we are
''''''sample the gyro for our angle calculation.  
Repeat
'
''''''Read Zeroed Values from Gyro''''''''
'Jason's code has a routine that zeros the gyro at startup, you can also get raw readings if you want.
    Gx := Gyro.GetRX
    Gy := Gyro.GetRY
    Gz := Gyro.GetRZ

'''''''Convert Gyro Values to Floating Point''''''''
    Gx    := FP32F.FFloat(Gx)
    Gy    := FP32F.FFloat(Gy)
    Gz    := FP32F.FFloat(Gz)
 ''''''Performs Math at Specific Time to Accumulate Gyro Angle''''''
 '''''' Gyro_Y_Angle = Gyro_Y_Angle + ((Gy/GyroLSB)*dT)
    Gy    := FP32F.FDiv(Gy,GyroLSB)
    Gy    := FP32F.FMul(Gy,GyroDT)
    Roll  := FP32F.FAdd(Roll,Gy)
'''''''Performs Math at Specific Time to Accumulate Gyro Angle''''''
 '''''' Gyro_X_Angle = Gyro_X_Angle + ((Gx/GyroLSB)*dT)
    Gx    := FP32F.FDiv(Gx,GyroLSB)
    Gx    := FP32F.FMul(Gx,GyroDT)
    Pitch := FP32F.FAdd(Pitch,Gx)

 ''''''Converts Roll and Pitch to Integers to Display in PST''''''
 'Converts Degrees into Integers, I take Roll and Pitch * 100 so that I can see 2 Decimal Places in the Result.
 'For Example 90 Degrees would Read 9000 Degrees
    RollI  := FP32F.FMul(OHund,Roll)
    PitchI := FP32F.FMul(OHund,Pitch)
'
'''''Rounds Roll and then Pitch and Converts to Integers''''''
    RollI  := FP32F.FRound(RollI)
    PitchI := FP32F.FRound(PitchI)
 ''''''Displays Roll and Pitch in PST Window''''''
 'If this section of code is expanded on it could throw of the 250Hz sample rate be carefull with adding to it.
    If DisplayCounter == 62     '62 is aprox. 1/4 of 250 this determines how often the data is sent to PST.Aprox. 4 times a second.
      PST.Clear
      PST.Char("X")
      PST.Char("=")
      PST.Char(" ")
      PST.Dec(RollI)
      PST.NewLine
      PST.Char("Y")
      PST.Char("=")
      PST.Char(" ")
      PST.Dec(PitchI)
      DisplayCounter := 0
    else
      DisplayCounter := DisplayCounter + 1
 ''''''Makes sure the repeat loop takes 4mS   
    waitcnt( constant(80_000_000 / 250) + LastTime )
    LastTime += constant(80_000_000 / 250)        